from data_code.gcp.secrets import get_secret
import psycopg2
import logging
import os

class Proxy():
    """Funcionalidades relacionadas al proxy para lograr comunicación con bases de datos.
    """
    def logprint_i(self,msg):
        logging.info(msg)
        print(msg)

    def raise_proxy(self, string_connection:str):
        self.logprint_i("Inicia levantacion del proxy.....")
        PROXYUP_COMMANDS = [
            f"rm -fr DTF && mkdir DTF && cd DTF && \
            wget https://dl.google.com/cloudsql/cloud_sql_proxy.linux.amd64 -O cloud_sql_proxy && ls -la cloud_sql_proxy && \
            chmod 755 cloud_sql_proxy && ls -la cloud_sql_proxy && pwd && ls -latrh && \
            nohup ./{string_connection} -ip_address_types=PRIVATE  & \&& \
            sleep 5 ",
        ]

        self.logprint_i("Inicia descarga proxy")
        for command in PROXYUP_COMMANDS:
            os.system(command)
        self.logprint_i("termina levantacion del proxy.....")

        self.logprint_i("proxy_up_executed")
    
    def shut_down_proxy(self):
        #TODO: Crear la baja del proxy para evitar vulnerabilidades.
        os.system("rm -fr DTF")
        print("\t\t\t\tShut down proxy")

class Database(Proxy):
    """_summary_

    Args:
        Proxy (Proxy): _description_
    """
    def __init__(self, host, port, database, secret_user, secret_pswd, project_id):
        self.host = host
        self.port = port
        self.database_name = database
        user = get_secret(secret_user,project_id)
        password = get_secret(secret_pswd,project_id)
        self.user = user
        self.password = password
        
    def get_connection(self):
        # Establecer la conexión con la base de datos
        connection = psycopg2.connect(
            host=self.host,
            port=self.port, 
            database=self.database_name,
            user=self.user,
            password=self.password
        )
        return connection
    
    def get_tables(self, connection:psycopg2):

        # Crear un cursor para ejecutar consultas
        cursor = connection.cursor()
        

        # Obtener los nombres de las tablas
        cursor.execute("""
                    SELECT table_name
                    FROM information_schema.tables
                    WHERE table_schema = 'public'
                    ORDER BY table_name
                    """)

        # Obtener los resultados
        filas = cursor.fetchall()

        # Procesar los resultados
        tablen="Tablenames"
        print(f"\n/ {tablen.upper():^20} \\")
        print("~~~~~~~~~~~~~~~~~~~~~~~~")
        
        for fila in filas:
            print(f"| {fila[0]:^20} |")
            print("________________________")
        print("\n")
        # Cerrar el cursor y la conexión
        cursor.close()
        
    def execute_query(self, connection:psycopg2, query:str):

        # Crear un cursor para ejecutar consultas
        cursor = connection.cursor()

        # Ejecutar una consulta de ejemplo
        cursor.execute(query)

        # Obtener los resultados de la consulta
        query_result = cursor.fetchall()

        # Procesar los resultados
        # for fila in filas:
        #     print(fila)

        # Cerrar el cursor y la conexión
        cursor.close()
        return query_result

    def close_connection(self, connection:psycopg2):
        # Cerrar el cursor y la conexión
        connection.close()




import apache_beam as beam
from data_code.gcp.sql import Database

class GetQuery(beam.DoFn):
    def __init__(self, host, port, database, query, string_conn, project_id, secret_user, secret_password):
        self.host=host
        self.port=port
        self.database=database
        self.query = query
        self.string_conn = string_conn
        self.project_id = project_id
        self.secret_user = secret_user
        self.secret_password = secret_password
        self.postgres = Database(self.host,self.port,self.database,self.secret_user,self.secret_password, self.project_id)
        self.conn = self.postgres.get_connection()

    def setup(self):
        self.postgres.raise_proxy(self.string_conn)

    def process(self, element, *args, **kwargs):
        
        
        query_result=self.postgres.execute_query(self.conn, query=self.query)
        
        yield query_result

    def teardown(self):
        self.postgres.close_connection(self.conn)
        self.postgres.shut_down_proxy()
        return super().teardown()




from google.cloud import bigquery
from google.cloud.exceptions import NotFound
import ast
import subprocess

def create_or_evaluate_bq_table(table_id:str,raw_parquet:str)->dict:
    """Verifica si existe la tabla y extrae los metadatos; en caso de que no exista la crea con base en el parquet de raw.

    Args:
        table (str): Nombre de la tabla.
        raw_parquet (str): Ruta del parquet.

    Returns:
        dict: Diccionario con la metadata de la tabla.
    """
    try:
        client = bigquery.Client()
        client.get_table(table_id.replace(':','.'))  # Make an API request.
        print(f"Table: {table_id} already exists.")
        metadata_table_bq = get_metadata(table_id)
        return metadata_table_bq
    except NotFound:
        print(f"Table: {table_id} not found. \nCreating it with parquet: {raw_parquet}")
        #TODO: bq load --autodetect --source_format=PARQUET <project_id>:<dataset>.<table_name> gs://<bucket_name>/<path_to_parquet_file>
        subprocess.run(["bq","load","--autodetect", "--source_format=PARQUET", table_id, raw_parquet], capture_output=True, shell=True).stdout
        print(f'{table_id} was created succesfully')
        metadata_table_bq = get_metadata(table_id)
        return metadata_table_bq

def get_metadata(table_id:str)->dict:
    """_summary_

    Args:
        table_id (str): _description_

    Returns:
        dict: _description_
    """
    metadata_table_bq = subprocess.run(["bq","show","--format=prettyjson", table_id], capture_output=True, shell=True).stdout
    metadata_table_bq = ast.literal_eval(metadata_table_bq.decode('UTF-8'))
    return metadata_table_bq